#include "atol_interface.h"

#pragma comment(linker,"/NODEFAULTLIB")
#pragma comment(linker,"/ENTRY:DllMain")
#pragma comment(linker,"/FILEALIGN:512")
#pragma comment(linker,"/MERGE:.data=.text")
#pragma comment(linker,"/MERGE:.rdata=.text")
#pragma comment(linker,"/SECTION:.text,ERWX")
#pragma comment(linker,"/IGNORE:4078")

// atol_console (C) 2005, Alexander G. Karpov [mrhx]

BOOL WINAPI DllMain(HINSTANCE, DWORD, LPVOID)
{
	return TRUE;
}

const char * ExportList [] =	// EXPORT LIST
{
	"console.write",			"write",
	"console.writenl",			"writenl",
	"console.read",				"read",
	"console.goto",				"cgoto",
	"console.color",			"color",
	"console.bgcolor",			"bgcolor",
	"console.title",			"title",
	"console.clear",			"clear",
	NULL
};

const char ** WINAPI atol_lib_install(ATOL_INTERFACE IAtol)
{
	return ExportList;
}

ATOL_ATOM WINAPI write(ATOL_INTERFACE IAtol, ATOL_LIST args, void *) // write on console.
{
	for (ATOL_LIST p = args; p; p = (*IAtol->list_next)(p))
		(*IAtol->print)( (*IAtol->list_item)(p) );

	return (*IAtol->list_item_atom)(args);
}

ATOL_ATOM WINAPI writenl(ATOL_INTERFACE IAtol, ATOL_LIST args, void *rsvd) // write on console.
{
	ATOL_ATOM res = write(IAtol, args, rsvd);
	(*IAtol->puts)("");
	return res;
}

ATOL_ATOM WINAPI read(ATOL_INTERFACE IAtol, ATOL_LIST args, void *) // read from console.
{
	char buf [1024];
	(*IAtol->gets)(buf, sizeof buf);
	return (*IAtol->atom)(buf);
}

ATOL_ATOM WINAPI title(ATOL_INTERFACE IAtol, ATOL_LIST args, void *) // set console title.
{
	SetConsoleTitle((*IAtol->list_item)(args));
	return (*IAtol->atom)("");
}

ATOL_ATOM WINAPI cgoto(ATOL_INTERFACE IAtol, ATOL_LIST args, void *) // Set console cursor position.
{
	COORD c;
	c.X = (*IAtol->atoi)( (*IAtol->list_item)(args) );
	c.Y = (*IAtol->atoi)( (*IAtol->list_item)( (*IAtol->list_next)(args) ) );

	SetConsoleCursorPosition(GetStdHandle(STD_OUTPUT_HANDLE), c);
	return (*IAtol->atom)("");
}

ATOL_ATOM WINAPI color(ATOL_INTERFACE IAtol, ATOL_LIST args, void *) // set console text color.
{
	static struct { const char * name; WORD attr; } Colors [] =
	{
		"black",			0,
		"blue",				FOREGROUND_BLUE,
		"green",			FOREGROUND_GREEN,
		"cyan",				FOREGROUND_BLUE | FOREGROUND_GREEN,
		"red",				FOREGROUND_RED,
		"magenta",			FOREGROUND_RED | FOREGROUND_BLUE,
		"brown",			FOREGROUND_RED | FOREGROUND_GREEN,
		"lightgray",		FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE,

		"darkgray",			FOREGROUND_INTENSITY,
		"lightblue",		FOREGROUND_BLUE | FOREGROUND_INTENSITY,
		"lightgreen",		FOREGROUND_GREEN | FOREGROUND_INTENSITY,
		"lightcyan",		FOREGROUND_BLUE | FOREGROUND_GREEN | FOREGROUND_INTENSITY,
		"lightred",			FOREGROUND_RED | FOREGROUND_INTENSITY,
		"lightmagenta",		FOREGROUND_RED | FOREGROUND_BLUE | FOREGROUND_INTENSITY,
		"yellow",			FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_INTENSITY,
		"white",			FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE | FOREGROUND_INTENSITY
	};

	ATOL_STRING name = (*IAtol->list_item)(args);
	int flag = 1;

	for (int i = 0; i < 16; ++i)
		if (!lstrcmp(name, Colors[i].name))
		{
			HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
			CONSOLE_SCREEN_BUFFER_INFO inf;

			GetConsoleScreenBufferInfo(hConsole, &inf);

			SetConsoleTextAttribute(hConsole, Colors[i].attr |
				(inf.wAttributes & (BACKGROUND_RED | BACKGROUND_GREEN | BACKGROUND_BLUE | BACKGROUND_INTENSITY))
			);
			--flag;
			break;
		}

	if (flag) (*IAtol->printf)("console.color: Unknown color name \"%s\"\n", name);

	return (*IAtol->list_item_atom)(args);
}

ATOL_ATOM WINAPI bgcolor(ATOL_INTERFACE IAtol, ATOL_LIST args, void *) // set console background color.
{
	static struct { const char * name; WORD attr; } Colors [] =
	{
		"black",			0,
		"blue",				BACKGROUND_BLUE,
		"green",			BACKGROUND_GREEN,
		"cyan",				BACKGROUND_BLUE | BACKGROUND_GREEN,
		"red",				BACKGROUND_RED,
		"magenta",			BACKGROUND_RED | BACKGROUND_BLUE,
		"brown",			BACKGROUND_RED | BACKGROUND_GREEN,
		"lightgray",		BACKGROUND_RED | BACKGROUND_GREEN | BACKGROUND_BLUE,

		"darkgray",			BACKGROUND_INTENSITY,
		"lightblue",		BACKGROUND_BLUE | BACKGROUND_INTENSITY,
		"lightgreen",		BACKGROUND_GREEN | BACKGROUND_INTENSITY,
		"lightcyan",		BACKGROUND_BLUE | BACKGROUND_GREEN | BACKGROUND_INTENSITY,
		"lightred",			BACKGROUND_RED | BACKGROUND_INTENSITY,
		"lightmagenta",		BACKGROUND_RED | BACKGROUND_BLUE | BACKGROUND_INTENSITY,
		"yellow",			BACKGROUND_RED | BACKGROUND_GREEN | BACKGROUND_INTENSITY,
		"white",			BACKGROUND_RED | BACKGROUND_GREEN | BACKGROUND_BLUE | BACKGROUND_INTENSITY
	};

	ATOL_STRING name = (*IAtol->list_item)(args);
	int flag = 1;

	for (int i = 0; i < 16; ++i)
		if (!lstrcmp(name, Colors[i].name))
		{
			HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
			CONSOLE_SCREEN_BUFFER_INFO inf;

			GetConsoleScreenBufferInfo(hConsole, &inf);

			SetConsoleTextAttribute(hConsole, Colors[i].attr |
				(inf.wAttributes & (FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE | FOREGROUND_INTENSITY))
			);
			--flag;
			break;
		}

	if (flag) (*IAtol->printf)("console.bgcolor: Unknown color name \"%s\"\n", name);

	return (*IAtol->list_item_atom)(args);
}

ATOL_ATOM WINAPI clear(ATOL_INTERFACE IAtol, ATOL_LIST, void *) // clear console.
{
	HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
	COORD c = GetLargestConsoleWindowSize(hConsole);
	COORD st = { 0, 0 };
	DWORD d;
	FillConsoleOutputCharacter(hConsole, ' ', c.X * (DWORD) c.Y, st, &d);
	FillConsoleOutputAttribute(hConsole, FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE,
		c.X * (DWORD) c.Y, st, &d);
	SetConsoleCursorPosition(hConsole, st); // set cursor position.
	return (*IAtol->atom)("");
}
